﻿Imports System.IO
Imports ThinkGeo.MapSuite.Core
Imports ThinkGeo.MapSuite.WebEdition

Partial Public Class RegionMap
    Inherits System.Web.UI.UserControl

    Private Const VWSTATE_REGIONID As String = "VwStRegionId"
    Public Property RegionId() As Integer
        Get
            If ViewState(VWSTATE_REGIONID) IsNot Nothing Then
                Return CType(ViewState(VWSTATE_REGIONID), Integer)
            Else
                Return -1
            End If
        End Get
        Set(ByVal value As Integer)
            ViewState(VWSTATE_REGIONID) = value
        End Set
    End Property

    Private ReadOnly Property DirMapPath() As String
        Get
            Dim mapPath As String = Server.MapPath("~/MapFiles")
            If ConfigurationManager.AppSettings("dirMapPath") IsNot Nothing Then
                mapPath = ConfigurationManager.AppSettings("dirMapPath")
            End If
            If Not mapPath.EndsWith("\") Then mapPath &= "\"
            Return mapPath
        End Get
    End Property

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        If Not IsPostBack Then
            InitMap()
            If RegionId <> -1 Then
                AddDirectoryMap(RegionId, True)
            End If
        End If
    End Sub

    Public Sub RefreshMap()
        ClearMap()
        AddDirectoryMap(RegionId, True)
    End Sub

    Private Sub InitMap()
		Dim latestVersion = "MapSuiteCore:3.1.124,Evaluation;WebEdition:3.1.124"
        If Not Map.Version.Equals(latestVersion, StringComparison.OrdinalIgnoreCase) Then
            Throw New Exception("If you see this exception, please update your WebEdition to latest version 3.1.124.")
        End If
	
        ctlRegionMap.MapBackground.BackgroundBrush = New GeoSolidBrush(GeoColor.FromHtml("#B3C6D4"))
        ctlRegionMap.CurrentExtent = New RectangleShape(-13939426.6371, 6701997.4056, -7812401.86, 2626987.386962)
        ctlRegionMap.MapUnit = GeographyUnit.Meter
        ctlRegionMap.MapTools.Logo.Enabled = False
        ctlRegionMap.MapTools.PanZoomBar.Enabled = True

        'Create the Yahoo map overlay
        Dim yahooMap As New YahooOverlay("Yahoo Map")
        yahooMap.JavaScriptLibraryUri = New Uri(ConfigurationManager.AppSettings("YahooUri"))
        yahooMap.YahooMapType = YahooMapType.Regular
        ctlRegionMap.BackgroundOverlay = yahooMap

        ctlRegionMap.DynamicOverlay.ClientCache.CacheId = "DirCache"
        ctlRegionMap.DynamicOverlay.ClientCache.Duration = TimeSpan.FromDays(1)

    End Sub

    Private Sub ClearMap()
        For layerIdx = ctlRegionMap.DynamicOverlay.Layers.Count - 1 To 0 Step -1
            ctlRegionMap.DynamicOverlay.Layers.Remove(layerIdx)
        Next
    End Sub

    Private Sub AddDirectoryMap(ByVal dirNum As Integer, ByVal highlight As Boolean)
        Dim dirLayer As ShapeFileFeatureLayer = Nothing
        For Each shapeLayer As ShapeFileFeatureLayer In ctlRegionMap.DynamicOverlay.Layers
            If shapeLayer.Name = "D" & dirNum.ToString() Then
                dirLayer = shapeLayer
                Exit For
            End If
        Next

        If dirLayer IsNot Nothing Then
            dirLayer.IsVisible = True
            If highlight Then
                dirLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush.Color = GeoColor.FromArgb(150, GeoColor.SimpleColors.LightGreen)
            Else
                dirLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush.Color = GeoColor.FromArgb(50, GeoColor.SimpleColors.PastelGreen)
            End If
            ctlRegionMap.CurrentExtent = GetMapBoundingBox()
        Else
            Dim dirNumStr As String = String.Format("{0:000000}", dirNum)

            If File.Exists(DirMapPath & dirNumStr & ".shp") Then
                dirLayer = New ShapeFileFeatureLayer(DirMapPath & dirNumStr & ".shp")
                dirLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen.Color = GeoColor.FromArgb(255, GeoColor.SimpleColors.Black)
                If highlight Then
                    dirLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush.Color = GeoColor.FromArgb(150, GeoColor.SimpleColors.LightGreen)
                Else
                    dirLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush.Color = GeoColor.FromArgb(50, GeoColor.SimpleColors.PastelGreen)
                End If
                dirLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.City1("DIRID")
                dirLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20
                dirLayer.Name = "D" & dirNum.ToString()

                Dim proj4 As New Proj4Projection(Proj4Projection.GetEpsgParametersString(4326), Proj4Projection.GetGoogleMapParametersString())
                dirLayer.FeatureSource.Projection = proj4

                ctlRegionMap.DynamicOverlay.Layers.Add("D" & dirNum.ToString(), dirLayer)
                ctlRegionMap.CurrentExtent = GetMapBoundingBox()
            End If
        End If

        ctlRegionMap.DynamicOverlay.Redraw()
    End Sub

    Private Function GetMapBoundingBox() As RectangleShape
        Dim result As RectangleShape = Nothing

        For Each l As ShapeFileFeatureLayer In ctlRegionMap.DynamicOverlay.Layers
            If l.IsVisible Then
                l.FeatureSource.Open()

                Dim feats As System.Collections.ObjectModel.Collection(Of Feature) = l.FeatureSource.GetAllFeatures(New String() {"DIRID"})

                Dim boundingBox As RectangleShape = feats(0).GetBoundingBox()
                System.Diagnostics.Debug.WriteLine("Layer Name: " & l.Name)
                System.Diagnostics.Debug.WriteLine("Layer: " & l.ShapePathFileName)
                System.Diagnostics.Debug.WriteLine("BoundingBox Information: ")
                System.Diagnostics.Debug.WriteLine("Width: " & boundingBox.Width)
                System.Diagnostics.Debug.WriteLine("Height: " & boundingBox.Height)
                System.Diagnostics.Debug.WriteLine("")
                If result Is Nothing Then
                    result = boundingBox
                Else
                    result.ExpandToInclude(boundingBox)
                End If

                l.FeatureSource.Close()
            End If
        Next

        Return result
    End Function

    Protected Sub AddAShapeClick(ByVal sender As Object, ByVal e As EventArgs)
        ctlRegionMap.DynamicOverlay.Layers(0).Open()
        Dim rectangle As RectangleShape = ctlRegionMap.DynamicOverlay.Layers(0).GetBoundingBox()
        ctlRegionMap.DynamicOverlay.Layers(0).Close()

        Dim lineLayer As New InMemoryFeatureLayer()
        lineLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1
        lineLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush = New GeoSolidBrush(GeoColor.FromArgb(60, GeoColor.StandardColors.Blue))
        lineLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20
        lineLayer.InternalFeatures.Add(New Feature(rectangle))
        ctlRegionMap.DynamicOverlay.Layers.Add(lineLayer)
        ctlRegionMap.DynamicOverlay.Redraw()
    End Sub
End Class